package ch.zhaw.projekt2.turnierverwaltung;

import javafx.collections.FXCollections;
import javafx.collections.ObservableList;

import java.io.File;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

public class Tournament implements Serializable {
    private String name;
    private Type type;
    private List<Participant> participants;
    private List<Place> places;



    public Tournament(String name, Type type) throws InvalidNameException, InvalidTypeException {
        if(!name.matches("[\\w]{1,20}")){
            throw new InvalidNameException("Invalid Name entered"); //TODO handle en logging.
        } else if(!Arrays.asList(Type.values()).contains(type)){
            throw new InvalidTypeException("Invalid Type selected"); //TODO handle en logging.
        }

        setName(name);
        setType(type);
        participants = new ArrayList<>();
        places = new ArrayList<>();
    }

    public void addParticipant(Participant newParticipant) throws ParticipantExistsException {
        participants.removeIf(participant -> participant.equals(newParticipant));
        participants.add(newParticipant);
    }

    public void removeParticipant(Participant participant) throws ParticipantNotExistsException {
        if(!participants.contains(participant)){
            throw new ParticipantNotExistsException("The given Participant is not part of this Tournament");
        }
        participants.remove(participant);
    }

    public ObservableList<Participant> getParticipants() {
        ObservableList<Participant> participantsObservable = FXCollections.observableArrayList();
        participantsObservable.addAll(participants);
        return participantsObservable;
    }

    public void addPlace(Place newPlace) throws PlaceExistsException {
        places.removeIf(place -> place.equals(newPlace));
        places.add(newPlace);
    }

    public void removePlace(Place place) throws PlaceNotExistsException {
        if(!places.contains(place)){
            throw new PlaceNotExistsException("The given Place is not part of this Tournament");
        }
        places.remove(place);
    }

    public ObservableList<Place> getPlaces() {
        ObservableList<Place> placesObservable = FXCollections.observableArrayList();
        placesObservable.addAll(places);
        return placesObservable;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public Type getType() {
        return type;
    }

    public void setType(Type type) {
        this.type = type;
    }

    public enum Type {
        KO("KO-System"), GROUPS("Gruppenspiele");

        private String name;

        private Type(String name) {
            this.name = name;
        }

        @Override
        public String toString() {
            return name;
        }

        public static ObservableList<Type> getObservableList(){
            ObservableList<Type> items = FXCollections.observableArrayList();
            items.addAll(values());
            return items;
        }
    }

    public class InvalidTypeException extends Exception {
        public InvalidTypeException() {
            super();
        }

        public InvalidTypeException(String errorMessage) {
            super(errorMessage);
        }

    }

    public class ParticipantExistsException extends Exception {
        public ParticipantExistsException() {
            super();
        }

        public ParticipantExistsException(String errorMessage) {
            super(errorMessage);
        }

    }

    public class ParticipantNotExistsException extends Exception {
        public ParticipantNotExistsException() {
            super();
        }

        public ParticipantNotExistsException(String errorMessage) {
            super(errorMessage);
        }

    }

    public class PlaceExistsException extends Exception {
        public PlaceExistsException() {
            super();
        }

        public PlaceExistsException(String errorMessage) {
            super(errorMessage);
        }

    }

    public class PlaceNotExistsException extends Exception {
        public PlaceNotExistsException() {
            super();
        }

        public PlaceNotExistsException(String errorMessage) {
            super(errorMessage);
        }

    }


}
